;===========================================================================================
; Trackbar.asm  GoAsm assembler
; Construction manuelle de Trackbar32 class
; Programm par AsmGges France
;===========================================================================================
; compile.bat:
; ..\..\bin\GoRC /r, res\trackbar.rc
; ..\..\bin\GoAsm trackbar.asm
; ..\..\bin\GoLink trackbar.obj res\trackbar.res user32.dll kernel32.dll gdi32.dll 
;                                                                     comctl32.dll
;===========================================================================================

; Inclusion des macros/quates/structures ncessaires au programme
; ----------------------------------------------------------------
  #include "..\..\inc\Windows.inc"
  #include "..\..\inc\all_api.inc"

  ID_FRANCE     = 17

  ID_GBOX       = 100
  ID_TRACKBARH1 = 101
  ID_TRACKBARH2 = 102
  ID_TRACKBARH3 = 103
  ID_TRACKBARV1 = 104
  ID_TRACKBARV2 = 105
  ID_TRACKBARV3 = 106
  ID_EDIT       = 107
  ID_SPIN       = 108

  UDM_SETRANGE32 = 046Fh
  UDM_SETPOS32   = 0471h

;===========================================================================================
; Dbut du code programme
; -----------------------
.code

  start:
    ; Obtenir le handle du programme
    ; ------------------------------
    invoke GetModuleHandle,0
    mov [wc.hInstance],eax

    ; Chargement d'une icne spcifique
    ; ---------------------------------
    invoke LoadIcon,[wc.hInstance],ID_FRANCE
    mov [wc.hIcon],eax

    ; Chargement d'un curseur systme
    ; -------------------------------
    invoke LoadCursor,0,IDC_ARROW
    mov [wc.hCursor],eax

    ; Obtenir la largeur en pixels de l'cran
    ; ---------------------------------------
    invoke GetSystemMetrics,SM_CXSCREEN
    sub eax,[WinW]
    shr eax,1
    mov [WinX],eax

    ; Valider la structure WNDCLASS
    ; -----------------------------
    invoke RegisterClass,addr wc

    ; Cration de la fenetre de l'application
    ; ---------------------------------------
    invoke CreateWindowEx,0,addr ClassName,addr Titre,WS_CAPTION+WS_POPUP+WS_SYSMENU \
                         +WS_MINIMIZEBOX,[WinX],[WinY],[WinW],[WinH],0,0,[wc.hInstance],0
    mov [hWnd],eax

    ; Affiche et rafraichit la fenetre
    ; --------------------------------
    invoke ShowWindow,[hWnd],SW_SHOWDEFAULT
    invoke UpdateWindow,[hWnd]

 ; Boucle des messages
 ; -------------------
 MessageLoop:
    invoke GetMessage,addr msg,0,0,0
    or eax,eax
    jz >ExitProgram
    invoke TranslateMessage,addr msg
    invoke DispatchMessage,addr msg
    jmp <MessageLoop

 ExitProgram:
    ; Fermeture de l'application
    ; --------------------------
    invoke ExitProcess,[msg.wParam]

;===========================================================================================
; Procdure principale Windows
;-------------------------------------------------------------------------------------------
WindowProc frame @hWnd, @uMsg, @wParam, @lParam
    uses ebx,esi,edi

    cmp D[@uMsg],WM_CLOSE
    jne >.wmcreate
        ; Boite  message fin d'application
        ; ---------------------------------
        invoke MessageBox,[@hWnd],addr MsgEnd,addr ClassName,MB_OK+MB_ICONINFORMATION

        ; Demande fin d'application
        ; -------------------------
        invoke PostQuitMessage,0
        jmp >>.exit

    .wmcreate:
    cmp D[@uMsg],WM_CREATE
    jne >.wmhscroll
        ; Assure que la bibliothque de liaison dynamique de contrle commune est charge
        ; -------------------------------------------------------------------------------
        mov D[icex.dwSize],sizeof INITCOMMONCONTROLSEX
        mov D[icex.dwICC],0FFFFh
        invoke InitCommonControlsEx,addr icex

        ; Dfinir une fonte de texte
        ; --------------------------
        invoke CreateFont,8,0,0,0,FW_NORMAL,0,0,0,ANSI_CHARSET,OUT_DEFAULT_PRECIS,\
                       CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH+FF_SWISS,\
                                                                      addr FontMS
        mov [hFontMS],eax                      ;font handle

        ; Construit tous les controles
        ; ----------------------------
        invoke CreateControls,[@hWnd]

        mov D[NewPos],10

        invoke TrackBarPos,[@hWnd]
        jmp >>.exit

    .wmhscroll:
    cmp D[@uMsg],WM_HSCROLL 
    je  >
        cmp D[@uMsg],WM_VSCROLL
        jne >>.defwindowproc
            :
            mov eax,[@wParam]
            and eax,0FFFFh
            cmp eax,TB_THUMBPOSITION
            jne >
                mov eax,[@wParam]
                shr eax,16
                mov [NewPos],eax
                invoke TrackBarPos,[@hWnd]
                jmp >>.exit
            :
            cmp eax,TB_THUMBTRACK
            jne >
                mov eax,[@wParam]
                shr eax,16
                mov [NewPos],eax
                invoke TrackBarPos,[@hWnd]
                jmp >>.exit
            :
            cmp eax,SB_LINEUP
            jne >.sblinedown
                cmp D[NewPos],0
                je  >
                    dec D[NewPos]
                :
                invoke TrackBarPos,[@hWnd]
                jmp >>.exit

            .sblinedown:
            cmp eax,SB_LINEDOWN
            jne >.exit
                cmp D[NewPos],20
                je  >
                    inc D[NewPos]
                :
                invoke TrackBarPos,[@hWnd]
                jmp >.exit

    .defwindowproc:
        ; Traitement par dfaut des autres messages Windows
        ; -------------------------------------------------
        invoke DefWindowProc,[@hWnd],[@uMsg],[@wParam],[@lParam]
        ret

    .exit:

    mov eax,0
    ret
endf

;===========================================================================================
CreateControls frame @hWnd

    ; Cration de la fentre-enfant GroupBox
    ; --------------------------------------
    invoke Dlg,addr Button,addr GbGges,[@hWnd],4,2,286,20,ID_GBOX,BS_GROUPBOX,0

    ;=======================================================================================
    ; Cration de la fentre-enfant TrackbarH1
    ; ----------------------------------------
    invoke Dlg,addr TrackBar,addr NoText,[@hWnd],16,30,100,34,ID_TRACKBARH1,\
                                                 WS_TABSTOP+TBS_AUTOTICKS,0
    ; Min/max TrackBarH1
    ; ------------------
    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARH1,TBM_SETRANGEMIN,0,0
    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARH1,TBM_SETRANGEMAX,0,20

    ;=======================================================================================
    ; Cration de la fentre-enfant TrackbarH2
    ; ----------------------------------------
    invoke Dlg,addr TrackBar,addr NoText,[@hWnd],16,80,100,34,ID_TRACKBARH2,\
                                         TBS_TOP+WS_TABSTOP+TBS_AUTOTICKS,0
    ; Min/max TrackBarH2
    ; ------------------
    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARH2,TBM_SETRANGEMIN,0,0
    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARH2,TBM_SETRANGEMAX,0,20

    ;=======================================================================================
    ; Cration de la fentre-enfant TrackbarH3
    ; ----------------------------------------
    invoke Dlg,addr TrackBar,addr NoText,[@hWnd],16,130,100,40,ID_TRACKBARH3,\
                                         TBS_BOTH+WS_TABSTOP+TBS_AUTOTICKS,0
    ; Min/max TrackBarH3
    ; ------------------
    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARH3,TBM_SETRANGEMIN,0,0
    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARH3,TBM_SETRANGEMAX,0,20

    ;=======================================================================================
    ; Cration de la fentre-enfant TrackbarV1
    ; ----------------------------------------
    invoke Dlg,addr TrackBar,addr NoText,[@hWnd],240,50,34,100,ID_TRACKBARV1,\
                                         TBS_VERT+WS_TABSTOP+TBS_AUTOTICKS,0
    ; Min/max TrackBarV1
    ; ------------------
    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARV1,TBM_SETRANGEMIN,0,0
    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARV1,TBM_SETRANGEMAX,0,20

    ;=======================================================================================
    ; Cration de la fentre-enfant TrackbarV2
    ; ----------------------------------------
    invoke Dlg,addr TrackBar,addr NoText,[@hWnd],190,50,34,100,ID_TRACKBARV2,\
                                 TBS_TOP+TBS_VERT+WS_TABSTOP+TBS_AUTOTICKS,0

    ; Min/max TrackBarV2
    ; ------------------
    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARV2,TBM_SETRANGEMIN,0,0
    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARV2,TBM_SETRANGEMAX,0,20

    ;=======================================================================================
    ; Cration de la fentre-enfant TrackbarV3
    ; ----------------------------------------
    invoke Dlg,addr TrackBar,addr NoText,[@hWnd],130,50,40,100,ID_TRACKBARV3,\
                                TBS_BOTH+TBS_VERT+WS_TABSTOP+TBS_AUTOTICKS,0

    ; Min/max TrackBarV3
    ; ------------------
    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARV3,TBM_SETRANGEMIN,0,0
    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARV3,TBM_SETRANGEMAX,0,20

    ;=======================================================================================
    ; Cration de la fentre-enfant Edit
    ; ----------------------------------
    invoke Dlg,addr Edit,addr NoText,[@hWnd],131,166,38,20,ID_EDIT,ES_CENTER+ES_NUMBER \
                                                         +ES_READONLY,WS_EX_CLIENTEDGE

    ;=======================================================================================
    ; Cration de la fentre-enfant Spin
    ; ----------------------------------
    invoke Dlg,addr Spin,addr NoText,[@hWnd],131,190,38,16,ID_SPIN,UDS_SETBUDDYINT \
                                           +UDS_AUTOBUDDY+UDS_ARROWKEYS+UDS_HORZ,0
    ; Min/max Spin
    ; ------------
    invoke SendDlgItemMessage,[@hWnd],ID_SPIN,UDM_SETRANGE32,0,20
    ;=======================================================================================

    ret
endf

;===========================================================================================
; Cration d'une fentre-enfant control class
;-------------------------------------------------------------------------------------------
; @Class    = adresse du nom de control class
; @AdrText  = adresse du control text
; @hParent  = handle de la fentre parent
; @X        = x-coordonne fentre-enfant
; @Y        = y-coordonne fentre-enfant
; @Xw       = largeur en pixels fentre-enfant
; @Yh       = hauteur en pixels fentre-enfant
; @Id       = identificateur du controle forme ID_XXX
; @Style    = complment window style de WS_CHILD+WS_VISIBLE
; @ExStyle  = window style tendu
;-------------------------------------------------------------------------------------------
; => eax = handle de la fentre-enfant
;-------------------------------------------------------------------------------------------
Dlg frame @Class, @AdrText, @hParent, @X, @Y, @Xw, @Yh, @Id, @Style, @ExStyle

    mov eax,WS_CHILD+WS_VISIBLE
    add eax,[@Style]
    invoke CreateWindowEx,[@ExStyle],[@Class],[@AdrText],eax,[@X],[@Y],[@Xw],[@Yh],\
                          [@hParent],[@Id],[wc.hInstance],NULL
    mov ebx,eax

    ; Valider la fonte standard MS Sans Serif pour le control
    ; -------------------------------------------------------
    invoke SendMessage,ebx,WM_SETFONT,[hFontMS],NULL
    mov eax,ebx

    ret
endf

;===========================================================================================
TrackBarPos frame @hWnd

    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARH1,TBM_SETPOS,-1,[NewPos]
    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARH2,TBM_SETPOS,-1,[NewPos]
    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARH3,TBM_SETPOS,-1,[NewPos]

    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARV1,TBM_SETPOS,-1,[NewPos]
    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARV2,TBM_SETPOS,-1,[NewPos]
    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARV3,TBM_SETPOS,-1,[NewPos]

    invoke SendDlgItemMessage,[@hWnd],ID_SPIN,UDM_SETPOS32,0,[NewPos]

    ret
endf

;===========================================================================================
; Dfinitions des datas chaines / variables initialises ou non initialises / structures
;-------------------------------------------------------------------------------------------
.data

  Titre       db ' TRACKBAR32 control',0
  ClassName   db ' AsmGges Win32',0
  MsgEnd      db ' GoAsm AsmGges France @2006  ',0
  GbGges      db 'AsmGges TrackBar32',0
  NoText      db '',0


  FontMS      db 'MS Sans Serif',0

  Edit        db 'EDIT',0
  Button      db 'BUTTON',0
  TrackBar    db 'msctls_trackbar32',0
  Spin        db 'msctls_updown32',0

  align 4

  WinX        dd 0
  WinY        dd 100
  WinW        dd 300
  WinH        dd 248

  NewPos      dd ?

  hWnd        dd ?
  hFontMS     dd ?

  wc          WNDCLASS             < 0, addr WindowProc, 0, 0, 0, 0, 0, \
                                     COLOR_BTNFACE+1, 0, addr ClassName >
  msg         MSG                  <>
  icex        INITCOMMONCONTROLSEX <>

;===========================================================================================
